@@ -5,32 +5,31 @@ import click
 from PyQt5.QtCore import QObject, QThreadPool
 from PyQt5.QtWidgets import QApplication
 
-from .worker import CASWorker
-
 
 class Cli(QObject):
-    def __init__(self, command, params, copy, parent=None):
+    def __init__(self, command, params, copy, worker, parent=None):
         super(Cli, self).__init__(parent)
 
         self.command = command
         self.params = params
         self.copy = copy
 
+        self.Worker = worker
+
+        self.threadpool = QThreadPool()
+
     def stop_thread(self):
         pass
 
-    def start_thread(self):
-        self.threadpool = QThreadPool()
-
     def call_worker(self):
         """
         Call worker, send command and params, and then start thread
         """
-        self.WorkerCAS = CASWorker(self.command, self.params, self.copy)
-        self.WorkerCAS.signals.output.connect(self.print_output)
-        self.WorkerCAS.signals.finished.connect(self.stop_thread)
+        worker = self.Worker(self.command, self.params, self.copy)
+        worker.signals.output.connect(self.print_output)
+        worker.signals.finished.connect(self.stop_thread)
 
-        self.threadpool.start(self.WorkerCAS)
+        self.threadpool.start(worker)
 
     def print_output(self, input_dict):
         """
@@ -74,7 +73,7 @@ DEFAULT_FLAGS = [
     click.option("--use-unicode", "-u", is_flag=True, default=False, help="Use unicode"),
     click.option("--line-wrap", "-l", is_flag=True, default=False, help="Use line wrap"),
     click.option("-c", "--copy", type=click.IntRange(1, 3),
-                 help="Copies the answer. 1 for exact_ans and 2 for approx_ans and 3 for a list of [exact_ans, "
+                 help="Copies the answer. 1 for exact_ans, 2 for approx_ans, and 3 for a list of [exact_ans, "
                       "approx_ans].")
 ]
 
@@ -190,15 +189,16 @@ def start():
 @add_options(DEFAULT_ARGUMENTS)
 @click.argument("params", nargs=-1)
 def deriv(params, **kwargs):
-    """Derive a function
+    """Derive a function.
 
-    Usage: caspy deriv EXPRESSION VARIABLE [ORDER] [AT_POINT]
+    Usage: caspy deriv EXPRESSION VARIABLE [ORDER] [AT_POINT] [FLAGS]
 
     \b
     Example(s):
     >>> caspy deriv x**x x
     >>> caspy deriv sin(1/x) x 3 pi
     """
+    from .qt_assets.tabs.derivative import DerivativeWorker
     default_params = ["x", "x", "1", None]
 
     validate_input_dict = validate_inputs(kwargs, default_params, params, "deriv")
@@ -217,7 +217,7 @@ def deriv(params, **kwargs):
     params_to_send = list_merge(default_params, list(params))
 
     to_send = [prefix + "deriv", params_to_send + options, kwargs["copy"]]
-    send_to_thread(to_send)
+    send_to_thread(to_send, DerivativeWorker)
 
 
 @main.command(cls=EncloseNegative)
@@ -229,13 +229,14 @@ def deriv(params, **kwargs):
 def integ(params, **kwargs):
     """Calculate definite and indefinite integrals of expressions.
 
-    Usage: caspy integ EXPRESSION VARIABLE {LOWER_BOUND UPPER_BOUND} [APPROXIMATE]
+    Usage: caspy integ EXPRESSION VARIABLE {LOWER_BOUND UPPER_BOUND} [APPROXIMATE] [FLAGS]
 
     \b
     Example(s):
     >>> caspy integ 1/sqrt(1-x**2) x -1 1
     >>> caspy integ x**x x -1 1 -A
     """
+    from .qt_assets.tabs.integral import IntegralWorker
     default_params = ["x", "x", None, None]
 
     validate_input_dict = validate_inputs(kwargs, default_params, params, "integ")
@@ -255,7 +256,44 @@ def integ(params, **kwargs):
     params_to_send = list_merge(default_params, list(params))
 
     to_send = [prefix + "integ", params_to_send + options, kwargs["copy"]]
-    send_to_thread(to_send)
+    send_to_thread(to_send, IntegralWorker)
+
+
+@main.command(cls=EncloseNegative)
+@add_options(DEFAULT_FLAGS)
+@add_options(DEFAULT_ARGUMENTS)
+@click.argument("params", nargs=-1)
+def sum(params, **kwargs):
+    """Calculate the summation of an expression.
+
+    Usage: caspy sum EXPRESSION VARIABLE START END [FLAGS]
+
+    \b
+    Example(s):
+    >>> caspy sum x**k/factorial(k) k 0 oo
+    >>> caspy sum k**2 k 1 m
+    """
+    from .qt_assets.tabs.summation import SummationWorker
+    default_params = ["k", "k", 1, 2]
+
+    validate_input_dict = validate_inputs(kwargs, default_params, params, "sum")
+    if list(validate_input_dict.keys())[0] == "error":
+        send_to_thread(validate_input_dict)
+        return
+
+    if kwargs["preview"]:
+        prefix = "prev_"
+        options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
+    else:
+        prefix = "calc_"
+        options = [kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"],
+                   kwargs["use_scientific"],
+                   kwargs["accuracy"]]
+
+    params_to_send = list_merge(default_params, list(params))
+
+    to_send = [prefix + "sum", params_to_send + options, kwargs["copy"]]
+    send_to_thread(to_send, SummationWorker)
 
 
 @main.command(cls=EncloseNegative)
@@ -265,13 +303,14 @@ def integ(params, **kwargs):
 def limit(params, **kwargs):
     """Calculate the limit of an expression.
 
-    Usage: caspy limit EXPRESSION VARIABLE AS_VARIABLE_IS_APPROACHING [SIDE]
+    Usage: caspy limit EXPRESSION VARIABLE AS_VARIABLE_IS_APPROACHING [SIDE] [FLAGS]
 
     \b
     Example(s):
     >>> caspy limit (1+1/(a*n))**(b*n) n oo
     >>> caspy limit n!**(1/n) n 0 -
     """
+    from .qt_assets.tabs.limit import LimitWorker
     default_params = ["x", "x", 0, "+-"]
 
     validate_input_dict = validate_inputs(kwargs, default_params, params, "limit")
@@ -290,7 +329,7 @@ def limit(params, **kwargs):
     params_to_send = list_merge(default_params, list(params))
 
     to_send = [prefix + "limit", params_to_send + options, kwargs["copy"]]
-    send_to_thread(to_send)
+    send_to_thread(to_send, LimitWorker)
 
 
 @main.command(cls=EncloseNegative)
@@ -306,13 +345,14 @@ def eq(params, **kwargs):
 
     Separate equation by either a space or a =, but not both.
 
-    Usage: eq LEFT_EXPRESSION RIGHT_EXPRESSION VARIABLE_TO_SOLVE_FOR [SOLVE_TYPE]
+    Usage: eq LEFT_EXPRESSION RIGHT_EXPRESSION VARIABLE_TO_SOLVE_FOR [SOLVE_TYPE] [FLAGS]
 
     \b
     Example(s):
     >>> caspy eq x**x 2 x
     >>> caspy eq sin(x)=1 x -st
     """
+    from .qt_assets.tabs.equations import EquationsWorker
     default_params = ["x", 0, "x"]
 
     if "=" in params[0]:
@@ -337,7 +377,7 @@ def eq(params, **kwargs):
     params_to_send = list_merge(default_params, list(params))
 
     to_send = [prefix + "normal_eq", params_to_send + options, kwargs["copy"]]
-    send_to_thread(to_send)
+    send_to_thread(to_send, EquationsWorker)
 
 
 @main.command(cls=EncloseNegative)
@@ -350,13 +390,14 @@ def diff_eq(params, **kwargs):
 
     Separate equation by either a space or a =, but not both.
 
-    Usage: diff-eq LEFT_EXPRESSION RIGHT_EXPRESSION FUNCTION_TO_SOLVE_FOR [HINT]
+    Usage: diff-eq LEFT_EXPRESSION RIGHT_EXPRESSION FUNCTION_TO_SOLVE_FOR [HINT] [FLAGS]
 
     \b
     Example(s):
     >>> caspy diff-eq f'(x) 1/f(x) f(x)
     >>> caspy diff-eq f''(x)+3*f'(x)=x**2 f(x)
     """
+    from .qt_assets.tabs.equations import EquationsWorker
     default_params = ["f(x)", "f(x)", "f(x)"]
 
     if "=" in params[0]:
@@ -385,47 +426,54 @@ def diff_eq(params, **kwargs):
 
     to_send = [prefix + "diff_eq", params_to_send + options, kwargs["copy"]]
 
-    send_to_thread(to_send)
+    send_to_thread(to_send, EquationsWorker)
 
 
 @main.command(cls=EncloseNegative)
 @add_options(DEFAULT_FLAGS)
 @add_options(DEFAULT_ARGUMENTS)
 @add_options(EQ_FLAGS)
+@click.option("--solve_type", "-st", is_flag=True, default=False,
+              help="Solve either a system of normal equations or a system of differential equations. Defualt if normal,"
+                   " set flag to solve a system of differential equations.")
 @click.argument("no_of_eq", type=int)
 def sys_eq(no_of_eq, **kwargs):
-    """Solves a system of equations.
+    """Solves a system of either normal or differential equations.
 
     Takes number of equations as argument, then will prompt user for all equations
 
-    Usage: sys-eq NO_OF_EQUATIONS
+    Usage: sys-eq NO_OF_EQUATIONS [SOLVE_TYPE] [FLAGS]
 
     \b
     Example(s):
     >>> caspy sys-eq 5
     >>> caspy sys-eq 3 -d Integers
     """
+    from .qt_assets.tabs.equations import EquationsWorker
+
+    if kwargs["solve_type"]:
+        solve_type = 2
+    else:
+        solve_type = 1
 
     equations = []
     for i in range(no_of_eq):
         equation = input(f"Enter equation number {i + 1} of {no_of_eq}: ")
         equations.append(equation)
 
-    variables = input(f"Enter variables to solve for separated by anything other than a-z, 0-9, and _: ")
+    variables = input(f"Enter variables to solve for separated by anything other than a-z, 0-9, (), and _: ")
 
     if kwargs["preview"]:
         prefix = "prev_"
-        options = [kwargs["domain"], kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
+        options = [kwargs["domain"], solve_type, kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"]]
     else:
         prefix = "calc_"
-        options = [kwargs["domain"], kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"],
+        options = [kwargs["domain"], solve_type, kwargs["output_type"], kwargs["use_unicode"], kwargs["line_wrap"],
                    kwargs["use_scientific"], kwargs["accuracy"], kwargs["verify_domain"]]
 
-    print(kwargs["domain"])
-
     to_send = [prefix + "system_eq", [equations] + [variables] + options, kwargs["copy"]]
 
-    send_to_thread(to_send)
+    send_to_thread(to_send, EquationsWorker)
 
 
 @main.command(cls=EncloseNegative)
@@ -434,12 +482,13 @@ def sys_eq(no_of_eq, **kwargs):
 def simp(expression, **kwargs):
     """Simplifies an expression.
 
-    Usage: simp EXPRESSION
+    Usage: simp EXPRESSION [FLAGS]
 
     \b
     Example(s):
     >>> caspy simp sin(x)**2+cos(x)**2
     """
+    from .qt_assets.tabs.simplify import SimpWorker
     expression = tuple([expression])
 
     default_params = ["x"]
@@ -457,7 +506,8 @@ def simp(expression, **kwargs):
 
     params_to_send = list_merge(default_params, list(expression))
     to_send = [prefix + "simp_exp", params_to_send + options, kwargs["copy"]]
-    send_to_thread(to_send)
+
+    send_to_thread(to_send, SimpWorker)
 
 
 @main.command(cls=EncloseNegative)
@@ -466,12 +516,13 @@ def simp(expression, **kwargs):
 def exp(expression, **kwargs):
     """Expandes an expression.
 
-    Usage: exp EXPRESSION
+    Usage: exp EXPRESSION [FLAGS]
 
     \b
     Example(s):
     >>> caspy exp (a+b-c)**3
     """
+    from .qt_assets.tabs.expand import ExpandWorker
     default_params = ["x"]
     expression = tuple([expression])
     validate_input_dict = validate_inputs(kwargs, default_params, expression, "exp")
@@ -488,7 +539,7 @@ def exp(expression, **kwargs):
 
     params_to_send = list_merge(default_params, list(expression))
     to_send = [prefix + "expand_exp", params_to_send + options, kwargs["copy"]]
-    send_to_thread(to_send)
+    send_to_thread(to_send, ExpandWorker)
 
 
 @main.command(cls=EncloseNegative)
@@ -508,13 +559,14 @@ def eval(expression, vars_sub, **kwargs):
     => 3**((3)+(5))
     => 6561
 
-    Usage: eval EXPRESSION [VARS_SUB ... ]
+    Usage: eval EXPRESSION [VARS_SUB ... ] [FLAGS]
 
     \b
     Example(s):
     >>> caspy eval exp(pi)+3/sin(6)
     >>> caspy eval 3**x x 3
     """
+    from .qt_assets.tabs.evaluate import EvaluateWorker
     default_params = ["1+1"]
     expression = tuple([expression])
     validate_input_dict = validate_inputs(kwargs, default_params, expression, "eval")
@@ -523,8 +575,7 @@ def eval(expression, vars_sub, **kwargs):
         return
 
     if len(vars_sub) % 2 != 0:
-        print(
-            "Variable substitution must consist of an even number of arguments, see 'eval --help' for more information")
+        print("Variable substitution must consist of an even number of arguments, see 'eval --help' for more information")
         return
 
     var_sub = ""
@@ -542,13 +593,13 @@ def eval(expression, vars_sub, **kwargs):
     params_to_send = list_merge(default_params, list(expression))
     to_send = [prefix + "eval_exp", params_to_send + [var_sub] + options, kwargs["copy"]]
 
-    send_to_thread(to_send)
+    send_to_thread(to_send, EvaluateWorker)
 
 
 @main.command(cls=EncloseNegative)
 @click.argument("number", type=int)
 @click.option("-c", "--copy", type=click.IntRange(1, 3),
-              help="Copies the answer. 1 for exact_ans and 2 for approx_ans and 3 for a list of [exact_ans, "
+              help="Copies the answer. 1 for exact_ans, 2 for approx_ans, and 3 for a list of [exact_ans, "
                    "approx_ans].")
 def pf(number, **kwargs):
     """Retreives the prime factors of an positive integer.
@@ -556,14 +607,15 @@ def pf(number, **kwargs):
     Note: exact_ans stores factors as dict: '{2: 2, 3: 1, 31: 1}'
     while approx_ans stores factors as string: '(2**2)*(3**1)*(31**1)'
 
-    Usage: pf NUMBER
+    Usage: pf NUMBER [FLAGS]
 
     \b
     Example(s):
     >>> caspy pf 372
     """
+    from .qt_assets.tabs.pf import PfWorker
     to_send = ["calc_pf", [number], kwargs["copy"]]
-    send_to_thread(to_send)
+    send_to_thread(to_send, PfWorker)
 
 
 @main.command(cls=EncloseNegative)
@@ -583,7 +635,8 @@ def web(website_index, list):
     """
 
     import json
-    with open("caspy3/data/websites.json", "r", encoding="utf8") as json_f:
+    import pkg_resources
+    with open(pkg_resources.resource_filename('caspy3', "data/websites.json"), "r", encoding="utf8") as json_f:
         json_data = json_f.read()
         web_list = json.loads(json_data)
 
@@ -602,7 +655,7 @@ def web(website_index, list):
         webbrowser.open(url)
 
 
-def send_to_thread(input_list):
+def send_to_thread(input_list, worker):
     def excepthook(exc_type, exc_value, exc_tb):
         tb = "".join(traceback.format_exception(exc_type, exc_value, exc_tb))
         print("error catched!:")
@@ -612,8 +665,7 @@ def send_to_thread(input_list):
     app = QApplication(sys.argv)
 
     try:
-        worker_thread = Cli(input_list[0], input_list[1], input_list[2])
-        worker_thread.start_thread()
+        worker_thread = Cli(input_list[0], input_list[1], input_list[2], worker)
         worker_thread.call_worker()
     except KeyError:
         print("Invalid number of arguments")
